import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
} from '@midwayjs/core';
import { DictDataService } from '../../service/system/dictData.service';
import { RF } from '../../response/CustomerResponse';
import { Page, Query } from '../../decorator/page.decorator';

@Controller('/dictData')
export class DictDataController {
  @Inject()
  service: DictDataService;

  @Post('/')
  async create(@Body() body) {
    const data = await this.service.create(body);
    return RF.success(data);
  }

  @Get('/')
  async query(@Query() query, @Page() page) {
    const result = await this.service.query(query, page);
    return RF.success(result);
  }

  @Get('/:id')
  async fetch(@Param('id') id: string) {
    const result = await this.service.fetch({ id });
    return RF.success(result);
  }

  @Post('/:id')
  async update(@Param('id') id: string, @Body() body) {
    const result = await this.service.update({ id }, body);
    return RF.success(result);
  }

  @Del('/:id')
  async delete(@Param('id') id: string) {
    await this.service.delete({ id });
    return RF.success();
  }
}
