import { Autoload, Init, Scope, ScopeEnum } from '@midwayjs/core';
import { InjectEntityModel } from '@midwayjs/typeorm';
import { Repository } from 'typeorm';
import { get } from 'lodash';
import * as bcrypt from 'bcryptjs';
import { Config } from '../../entity/system/config.entity';
import { DictData } from '../../entity/system/dictData.entity';
import { DictType } from '../../entity/system/dictType.entity';
import { Menus } from '../../entity/system/menus.entity';
import { Role } from '../../entity/system/role.entity';
import { Admin } from '../../entity/system/admin.entity';
@Autoload()
@Scope(ScopeEnum.Singleton)
export class InitService {
  @InjectEntityModel(Admin)
  adminModel: Repository<Admin>;
  @InjectEntityModel(Role)
  roleModel: Repository<Role>;
  @InjectEntityModel(Menus)
  menusModel: Repository<Menus>;
  @InjectEntityModel(DictType)
  dictTypeModel: Repository<DictType>;
  @InjectEntityModel(DictData)
  dictDataModel: Repository<DictData>;
  @InjectEntityModel(Config)
  configModel: Repository<Config>;


  @Init()
  async init() {
    this.adminUser();
    this.initRole();
    this.initMenus();
    this.initDict();
    this.initConfig();
  }
  async initConfig() {
    const data: any = { logo: [], name: '平台名' };
    const num = await this.configModel.count();
    if (num > 0) return;
    await this.configModel.insert(data);
  }

  async adminUser() {
    const salt = bcrypt.genSaltSync(10);
    const hash = bcrypt.hashSync('1qaz2wsx', salt);
    const data: any = {
      account: 'sadmin',
      password: hash,
      nick_name: '系统管理员',
      is_super: '0',
    };
    
    const query: any = { is_super: '0' }
    const is_exist = await this.adminModel.count({ where: query });
    if (!is_exist) return await this.adminModel.insert(data);
    return;
  }

  async initRole() {
    const num = await this.roleModel.count();
    if (num > 0) return;
    const datas: any = [{ name: '管理员', code: 'admin' }];
    await this.roleModel.insert(datas);
  }
  async initMenus() {
    const num = await this.menusModel.count();
    if (num > 0) return;
    const datas: any = [
      {
        name: '首页',
        order_num: 1,
        path: '/',
        component: '/home/index',
        type: '1',
        route_name: 'home',
      },
    ];
    await this.menusModel.insert(datas);
    // 系统设置
    const sd = {
      name: '系统设置',
      path: '/system',
      order_num: 2,
      type: '0',
      route_name: 'system',
      is_default: '0',
    };
    const rsd = await this.menusModel.insert(sd);
    const smId = get(rsd, 'identifiers.0.id')
    const systemMenus = [
      {
        name: '平台设置',
        parent_id: smId,
        order_num: 1,
        path: '/system/config',
        component: '/system/config/index',
        type: '1',
        route_name: 'system_config',
        is_default: '0',
      },
      {
        name: '目录设置',
        parent_id: smId,
        order_num: 1,
        path: '/system/menus',
        component: '/system/menus/index',
        type: '1',
        route_name: 'system_menus',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'update',
          },
          {
            zh: '添加下一级',
            code: 'addNext',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
        is_default: '0',
      },
      {
        name: '角色管理',
        parent_id: smId,
        order_num: 2,
        path: '/system/role',
        component: '/system/role/index',
        type: '1',
        route_name: 'system_role',
        is_default: '0',
        config: [
          {
            zh: '添加',
            code: 'add',
          },
          {
            zh: '修改',
            code: 'update',
          },
          {
            zh: '使用',
            code: 'toAbled',
          },
          {
            zh: '禁用',
            code: 'toDisabled',
          },
          {
            zh: '删除',
            code: 'delete',
          },
        ],
      },
      {
        name: '字典管理',
        parent_id: smId,
        order_num: 3,
        path: '/system/dict',
        component: '/system/dict/index',
        type: '1',
        route_name: 'system_dict',
        is_default: '0',
      },
      {
        name: '字典数据',
        parent_id: smId,
        order_num: 4,
        path: '/system/dictData',
        component: '/system/dictData/index',
        type: '2',
        route_name: 'system_dict_data',
        is_default: '0',
      },
    ];
    await this.menusModel.insert(systemMenus);

    // 用户管理
    const ud = {
      name: '用户管理',
      order_num: 3,
      path: '/user',
      type: '0',
      route_name: 'user',
      is_default: '1',
    };
    const rud = await this.menusModel.insert(ud);
    const umId = get(rud, 'identifiers.0.id')
    const userMenus = [
      {
        name: '管理员用户',
        parent_id: umId.toString(),
        order_num: 1,
        path: '/user/admin',
        component: '/user/admin/index',
        type: '1',
        route_name: 'user_admin',
      },
      {
        name: '平台用户',
        parent_id: umId.toString(),
        order_num: 2,
        path: '/user/user',
        component: '/user/user/index',
        type: '1',
        route_name: 'user_user',
      },
    ];
    await this.menusModel.insert(userMenus);

    const password = {
      name: '账号管理',
      order_num: 999,
      path: '/acccount',
      component: '/acccount/index',
      type: '1',
      route_name: 'acccount',
    };
    await this.menusModel.insert(password);
  }
  async initRoleMenu(admin: Admin) { }

  async initDict() {
    const num = await this.dictTypeModel.count();
    if (num > 0) return;
    const isUseType: any = [{ title: '是否使用', code: 'isUse', is_use: '0' }];
    const isUseData: any = [
      { code: 'isUse', label: '使用', value: '0', sort: 1, is_use: '0' },
      { code: 'isUse', label: '禁用', value: '1', sort: 2, is_use: '0' },
    ];
    await this.dictTypeModel.insert(isUseType);
    await this.dictDataModel.insert(isUseData);
  }
}
