import { Provide } from "@midwayjs/core";
import { InjectDataSource } from "@midwayjs/typeorm";
import { get, lowerCase, replace } from "lodash";
import { DataSource, ObjectLiteral, Repository } from "typeorm";
import { ServiceError } from "../../error/CustomerError.error";
import { ErrorCode } from "../../error/Codes";

@Provide()
export class CommonService {
  @InjectDataSource('default')
  defaultDataSource: DataSource;
  /**
   * 查询列表
   * @param model 表实体
   * @param query 查询条件
   * @param page 分页设置
   * @param others 其他设置
   * @returns data 列表数据
   * @returns total 总数
   */
  async search(model: Repository<ObjectLiteral>, query: object = {}, page: object = {}, others: object = {}) {
    const searchObject: any = {}
    const countObject: any = {}
    if (Object.keys(query).length > 0) {
      searchObject.where = query
      countObject.where = query
    }
    if (Object.keys(page).length > 0) {
      const skip = get(page, 'skip', 0)
      if (skip >= 0) searchObject.skip = skip
      const take = get(page, 'limit', 0)
      if (take > 0) searchObject.take = take
    }
    if (Object.keys(others).length > 0) {
      for (const key in others) {
        searchObject[key] = others[key]
      }
    }
    const data = await model.find(searchObject)
    const total = await model.count(countObject)
    return { data, total };
  }
  /**
   * 单查询
   * @param model 表实体
   * @param query 查询条件
   * @returns data 数据
   */
  async getOne(model: Repository<ObjectLiteral>, query: object) {
    const data = await model.findOne({ where: query });
    return data;
  }

  /**
   * 根据表名查询表实例
   * @param tableName 表名
   * @returns model 表的实例
   */
  getModel(tableName: string): Repository<ObjectLiteral> {
    const entitys = this.defaultDataSource.entityMetadatas;
    const entity = entitys.find(f => replace(lowerCase(f.name), " ", '') === replace(lowerCase(tableName), " ", ''))
    if (!entity) throw new ServiceError(ErrorCode.ENTITY_NOT_FOUND)
    const entityClass = entity.target;
    const model = this.defaultDataSource.getRepository(entityClass)
    return model
  }
}