import { Provide } from "@midwayjs/core";
import { InjectEntityModel } from "@midwayjs/typeorm";
import { Repository } from "typeorm";
import { Question } from "../entity/question.entity";
import { get, isNull, isString, isUndefined } from "lodash";

@Provide()
export class QuestionService {
  @InjectEntityModel(Question)
  model: Repository<Question>;

  async query(query: object = {}, meta: any = {}) {
    let skip = get(meta, 'skip', 0);
    let limit = get(meta, 'limit', 0);
    const order = get(meta, 'order', {});
    const selects = get(meta, 'selects', []);
    const builder = await this.model.createQueryBuilder();
    if (query) builder.where(query);
    if (selects.length > 0) {
      // 字段是直接传来的,正常限制,需要加上model的name.否则会导致什么字段都没有
      const modelName = this.model.metadata.name;
      builder.select(selects.map(i => `${modelName}.${i}`));
    }
    // 组织查询顺序
    let orderObject: any = {};
    // 如果有自定义顺序,则按照自定义顺序来, 没有自定义顺序,默认按创建时间的desc查询
    if (Object.keys(order).length > 0) {
      for (const column in order) orderObject[column] = order[column];
    } else orderObject = { question_id: 'DESC' };
    // 分页
    if (isString(skip)) {
      skip = parseInt(skip);
      if (isFinite(skip)) builder.skip(skip);
    } else if (isFinite(skip)) builder.skip(skip);
    if (isString(limit)) {
      limit = parseInt(limit);
      if (isFinite(limit)) builder.take(limit);
    } else if (isFinite(limit)) builder.take(limit);
    // 排序
    builder.orderBy(orderObject);
    const data = await builder.getMany();
    const total = await builder.getCount();
    return { data, total };
  }

  async create(data: object) {
    const result = await this.model.insert(data);
    const id = get(result, 'identifiers.0.id');
    // 没有id估计是出错了
    if (!id) return;
    const createData = await this.fetch({ id });
    return createData;
  }

  async fetch(query: object) {
    const builder = this.model.createQueryBuilder();
    builder.where(query)
    const result = await builder.getOne();
    return result;
  }

  /**修改，单修改/多修改是统一修改为 */
  async update(query: object = {}, data: object) {
    // 没有范围的修改不允许执行
    if (Object.keys(query).length <= 0) return;
    // 处理数据, 只将是本表的字段拿出来保存
    const columns = this.model.metadata.columns;
    /**将array的列设置 转换为object,以便query使用*/
    const columnsObject = {};
    // 整理成object
    for (const c of columns) columnsObject[c.propertyName] = c.type.toString();
    const updateData = {};
    const notDealColumn = ['created_time', 'update_time', 'data_owner', '__v'];
    for (const column in columnsObject) {
      if (notDealColumn.includes(column)) continue;
      const val = data[column];
      if (isNull(val) || isUndefined(val)) continue;
      updateData[column] = val;
    }
    await this.model.update(query, updateData);
  }

}